﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Windows.Forms;

namespace MazeMaker
{
    public class CellType
    {
        private static CellType[] cellTypeByID = new CellType[byte.MaxValue + 1];

        public string name;
        public string uri;
        public Bitmap bitmap;

        private byte id;

        // Unfortuntely, C# doesn't seem to allow static accessors
        // I would have made this a static indexer, it woulda been pretty nifty.
        public static CellType FromID(byte ID)
        {
            return cellTypeByID[ID];
        }

        public byte ID
        {
            get { return id; }
        }

        // ID should be unique and non-zero
        // TODO?: Allow auto-pick of ID
        public CellType(string uri, byte ID) : this(uri, null, ID) { }
        public CellType(string uri, string name, byte ID)
        {
            if(ID == 0)
            {
                throw new ArgumentOutOfRangeException("ID",
                    String.Format("ID for CellType \"{0}\" was zero. CellType IDs must be non-zero, because ID zero is reserved for the CellType of null.",
                                  name));
            }

            if(CellType.FromID(ID) != null)
            {
                throw new ArgumentException(
                    String.Format("ID {0} for CellType \"{1}\" is already taken by \"{2}\". CellType IDs must be unique.",
                                  ID, name, CellType.FromID(ID).name),
                    "ID");
            }
            cellTypeByID[ID] = this;

            this.name = (name == null) ? uri : name;
            this.uri = uri;
            this.id = ID;
            this.bitmap = new Bitmap(uri);
        }

        public override string ToString()
        {
            return String.Format("{1} (#{0}: {2})", id, name, uri);
        }
    }

/*    public class Cell
    {
        public const int width = 16;
        public const int height = 16;
        //public const string blockEmptyURL = "block_empty.bmp";
        //public const string blockWallURL = "block_wall.bmp";

        //public static Bitmap blockEmptyBitmap;  // TODO: make BlockType classes
        //public static Bitmap blockWallBitmap;

        public int x;
        public int y;
//        public CellType type;

//            public bool Visible
//        {
//            get
//            {
//                return pictureBox.Visible;
//            }
//            set
//            {
//                pictureBox.Visible = value;
//            }
//        }

//        public string ImageURL
//        {
//            get
//            {
//                return "";
//                //testCell.PictureBox.LoadAsync(blockWallURL);
//                //                    return pictureBox.Visible;
//            }
//            set
//            {
////                    pictureBox.Visible = value;
//            }
//        }

        public Cell(int x, int y) : this(x, y, null) {}
        public Cell(int x, int y, CellType type)
        {
            this.x = x;
            this.y = y;
            this.type = type;

            //Setup static fields
            //if(blockEmptyBitmap == null)
            //    blockEmptyBitmap = new Bitmap(blockEmptyURL);

            //if(blockWallBitmap == null)
            //    blockWallBitmap = new Bitmap(blockWallURL);
        }
    }
*/
}
