﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MazeMaker
{
    class GridFileException : FormatException
    {
        protected readonly string uri;
        public const string defaultGridFileExceptionMessage =
            "The file is not a valid Maze Maker Grid file.";

        public string Uri
        {
            get
            {
                return uri;
            }
        }

        public GridFileException()
            : base(defaultGridFileExceptionMessage)
        {
        }

        public GridFileException(string uri)
            : base(defaultGridFileExceptionMessage)
        {
            this.uri = uri;
        }

        public GridFileException(string uri, string message)
            : base(message)
        {
            this.uri = uri;
        }

        public GridFileException(Exception innerException)
            : base(defaultGridFileExceptionMessage, innerException)
        {
        }

        public GridFileException(string uri, Exception innerException)
            : base(defaultGridFileExceptionMessage, innerException)
        {
            this.uri = uri;
        }

        public GridFileException(string uri, string message, Exception innerException)
            : base(message, innerException)
        {
            this.uri = uri;
        }
    }

    class TypeIDException : GridFileException
    {
        public const string defaultTypeIDExceptionMessage =
            defaultGridFileExceptionMessage + " (Unknown Type ID)";

        public TypeIDException()
            : base(null, defaultTypeIDExceptionMessage)
        {
        }

        public TypeIDException(string uri)
            : base(uri, defaultTypeIDExceptionMessage)
        {
        }

        public TypeIDException(string uri, string message)
            : base(uri, message)
        {
        }

        public TypeIDException(Exception innerException)
            : base(null, defaultTypeIDExceptionMessage, innerException)
        {
        }

        public TypeIDException(string uri, Exception innerException)
            : base(uri, defaultTypeIDExceptionMessage, innerException)
        {
        }

        public TypeIDException(string uri, string message, Exception innerException)
            : base(uri, message, innerException)
        {
        }
    }

    class FutureVersionException : GridFileException
    {
        public const string defaultFutureVersionExceptionMessage =
            "The file was created by newer version of Map Maker.";

        public FutureVersionException()
            : base(null, defaultFutureVersionExceptionMessage)
        {
        }

        public FutureVersionException(string uri)
            : base(uri, defaultFutureVersionExceptionMessage)
        {
        }

        public FutureVersionException(string uri, string message)
            : base(uri, message)
        {
        }

        public FutureVersionException(Exception innerException)
            : base(null, defaultFutureVersionExceptionMessage, innerException)
        {
        }

        public FutureVersionException(string uri, Exception innerException)
            : base(uri, defaultFutureVersionExceptionMessage, innerException)
        {
        }

        public FutureVersionException(string uri, string message, Exception innerException)
            : base(uri, message, innerException)
        {
        }
    }

    class UnknownVersionException : GridFileException
    {
        public const string defaultUnknownVersionExceptionMessage =
            defaultGridFileExceptionMessage + " (Unknown Version)";

        public UnknownVersionException()
            : base(null, defaultUnknownVersionExceptionMessage)
        {
        }

        public UnknownVersionException(string uri)
            : base(uri, defaultUnknownVersionExceptionMessage)
        {
        }

        public UnknownVersionException(string uri, string message)
            : base(uri, message)
        {
        }

        public UnknownVersionException(Exception innerException)
            : base(null, defaultUnknownVersionExceptionMessage, innerException)
        {
        }

        public UnknownVersionException(string uri, Exception innerException)
            : base(uri, defaultUnknownVersionExceptionMessage, innerException)
        {
        }

        public UnknownVersionException(string uri, string message, Exception innerException)
            : base(uri, message, innerException)
        {
        }
    }

    class UnsupportedDimentionsException : GridFileException
    {
        public const string defaultUnsupportedDimentionsExceptionMessage =
            defaultGridFileExceptionMessage + " (Unsupported Dimentions)";

        public UnsupportedDimentionsException()
            : base(null, defaultUnsupportedDimentionsExceptionMessage)
        {
        }

        public UnsupportedDimentionsException(string uri)
            : base(uri, defaultUnsupportedDimentionsExceptionMessage)
        {
        }

        public UnsupportedDimentionsException(string uri, string message)
            : base(uri, message)
        {
        }

        public UnsupportedDimentionsException(Exception innerException)
            : base(null, defaultUnsupportedDimentionsExceptionMessage, innerException)
        {
        }

        public UnsupportedDimentionsException(string uri, Exception innerException)
            : base(uri, defaultUnsupportedDimentionsExceptionMessage, innerException)
        {
        }

        public UnsupportedDimentionsException(string uri, string message, Exception innerException)
            : base(uri, message, innerException)
        {
        }
    }

    class EOFInHeaderException : GridFileException
    {
        public const string defaultEOFInHeaderExceptionMessage =
            defaultGridFileExceptionMessage + " (Reached End Of File Early)";

        public EOFInHeaderException()
            : base(null, defaultEOFInHeaderExceptionMessage)
        {
        }

        public EOFInHeaderException(string uri)
            : base(uri, defaultEOFInHeaderExceptionMessage)
        {
        }

        public EOFInHeaderException(string uri, string message)
            : base(uri, message)
        {
        }

        public EOFInHeaderException(Exception innerException)
            : base(null, defaultEOFInHeaderExceptionMessage, innerException)
        {
        }

        public EOFInHeaderException(string uri, Exception innerException)
            : base(uri, defaultEOFInHeaderExceptionMessage, innerException)
        {
        }

        public EOFInHeaderException(string uri, string message, Exception innerException)
            : base(uri, message, innerException)
        {
        }
    }

    class EOFInMapDataException : GridFileException
    {
        public const string defaultEOFInMapDataExceptionMessage =
            "The file is incomplete.";

        public EOFInMapDataException()
            : base(null, defaultEOFInMapDataExceptionMessage)
        {
        }

        public EOFInMapDataException(string uri)
            : base(uri, defaultEOFInMapDataExceptionMessage)
        {
        }

        public EOFInMapDataException(string uri, string message)
            : base(uri, message)
        {
        }

        public EOFInMapDataException(Exception innerException)
            : base(null, defaultEOFInMapDataExceptionMessage, innerException)
        {
        }

        public EOFInMapDataException(string uri, Exception innerException)
            : base(uri, defaultEOFInMapDataExceptionMessage, innerException)
        {
        }

        public EOFInMapDataException(string uri, string message, Exception innerException)
            : base(uri, message, innerException)
        {
        }
    }
}
